"""Common stop words for text processing."""

# Common English stop words to filter out in text processing
DEFAULT_STOP_WORDS = {
    "a",
    "an",
    "the",
    "and",
    "or",
    "but",
    "is",
    "are",
    "was",
    "were",
    "in",
    "on",
    "at",
    "to",
    "for",
    "with",
    "by",
    "about",
    "of",
    "from",
    "as",
    "that",
    "this",
    "these",
    "those",
    "it",
    "its",
    "them",
    "they",
    "their",
    "he",
    "she",
    "his",
    "her",
    "him",
    "we",
    "our",
    "you",
    "your",
    "not",
    "be",
    "been",
    "being",
    "have",
    "has",
    "had",
    "do",
    "does",
    "did",
    "can",
    "could",
    "will",
    "would",
    "shall",
    "should",
    "may",
    "might",
    "must",
    "when",
    "where",
    "which",
    "who",
    "whom",
    "whose",
    "why",
    "how",
}
